from xfutils.audit_constant import *
from xfutils.audit_utils import get_list_of_maps_diff, get_map_diff, get_data_key


def get_playbook_update_data(audit_log, audit_data):
    old_data = audit_data.get('oldData')
    new_data = audit_data.get('newData')
    old_steps = old_data.get("steps", [])
    playbook_name = new_data.get(NAME_FIELD)
    new_steps = new_data.get("steps", [])

    diff_keys = ["name", "arguments", "stepType"]
    mandatory_keys = ["name", "stepType"]
    exclude_keys = ["routes", "modifyDate", "modifyUser", "steps"]

    steps_map_diff = get_list_of_maps_diff(old_steps, new_steps, IRI, None, diff_keys, mandatory_keys)
    steps_map_diff_result = {}

    for key, steps in steps_map_diff.items():
        temp_steps = []
        if key == "UPDATE":
            for step in steps:
                step_data = {}
                step_name = step.get(NAME_FIELD)
                step_type = step.get("stepType")
                step_type_name = step_type.get('displayName')

                arguments_map = step.get("arguments", None)
                step_data["name"] = step_name
                step_data["stepType"] = step_type_name

                if arguments_map:
                    step_data["arguments"] = arguments_map

                temp_steps.append(step_data)
        else:
            for step in steps:
                step_name_type = {}
                step_name = step.get(NAME_FIELD)
                step_type = step.get("stepType")
                step_type_name = step_type.get('displayName')
                step_name_type["name"] = step_name
                step_name_type["stepType"] = step_type_name

                temp_steps.append(step_name_type)

        operation_verb = key + "D"
        if key in SECOND_FORM_MAP:
            operation_verb = SECOND_FORM_MAP[key]

        steps_map_diff_result[operation_verb] = temp_steps

    # Compute the diff between old and new data
    diff = {}
    if old_data != new_data:
        diff = get_map_diff(old_data, new_data, exclude_keys, None, mandatory_keys)

    updated = steps_map_diff_result.get("Updated", [])
    created = steps_map_diff_result.get("Created", [])
    deleted = steps_map_diff_result.get("Deleted", [])

    if not diff and not updated and not created and not deleted:
        return None

    diff["steps"] = steps_map_diff_result

    return diff


def get_generic_change_data(record_log, audit_data):
    data_key = get_data_key(record_log.get('operation'))
    return audit_data.get(data_key, audit_data.get('data', audit_data))


def get_change_data_func(entity_type, operation):
    # todo generic change data should be operation based
    return change_data_mapping.get(entity_type, {}).get(operation, get_generic_change_data)


change_data_mapping = {
    "playbooks": {
        "publish": get_playbook_update_data,
    },
    "generic":{
        "create": get_generic_change_data
    }
}