get_password() {
    local pgpw_1
    local pgpw_2
    printf "Enter a password to be used for the Agent database: "
    stty -echo
    read pgpw_1
    printf "\n"
    if [[ -z $pgpw_1 ]]; then
        printf "Password string may not be empty. Please try again.\n"
        return
    fi
    if [[ ! $pgpw_1 =~ .*[a-z].* ]] || [[ ! $pgpw_1 =~ .*[A-Z].* ]] || [[ ! $pgpw_1 =~ .*[0-9].* ]] || [[ ! ${#pgpw_1} -ge 10 ]] || [[ ! ${#pgpw_1} -le 64 ]]; then
        printf "Passwords must contain one lowercase letter, one uppercase letter, and one numerical digit, and must be between 10 and 64 characters long. Please ensure your password meets these requirements and try again.\n"
        return
    fi
    printf "Re-enter chosen password: "
    read pgpw_2
    stty echo
    printf "\n"
    if [[ $pgpw_1 != $pgpw_2 ]]; then
        printf "Password strings must match. Please try again.\n"
        return
    else
        pgpw=$pgpw_1
        return
    fi
}

create_docker_compose_file() {
    local dc_content="@dockercompose@"
    printf "$dc_content" > "$composefile"
}

populate_extra_hosts() {
    sed -i.bak "66i\\
    extra_hosts:
" "$composefile"
    sed -i.bak "67i\\
      - \"$rmq_hostname:$rmq_ip\"
" $composefile

  rm -f "$composefile.bak"
}

get_ini_value() {
    local key=$1
    # Extract value using grep and sed. Adjust for different .ini file structures as needed.
    grep -oP "(?<=^$key=).*" "$INI_FILE"
}

main() {
    local rmq_ip="69.167.115.175"
    local rmq_hostname="uat-agent-rmq.pool1.ca-west-1.xf.fortisoar.forticloud.com"
    local pgpw
    local pguser
    if [ -z "$1" ]; then
      pgpw=""
      while [[ -z $pgpw ]]
      do
        get_password
      done
    elif [[ $1 == *.ini ]]; then
      INI_FILE="$1"
      pgpw=$(get_ini_value "pgpw")
      if [ -z $pgpw ]; then
        echo "No variable named 'pgpw' present in supplied .ini file. Exiting XFabric Agent Installer"
        exit 1
      elif [[ ! $pgpw =~ .*[a-z].* ]] || [[ ! $pgpw =~ .*[A-Z].* ]] || [[ ! $pgpw =~ .*[0-9].* ]] || [[ ! ${#pgpw} -ge 10 ]] || [[ ! ${#pgpw} -le 64 ]]; then
        echo "Passwords must contain one lowercase letter, one uppercase letter, and one numerical digit, and must be between 10 and 64 characters long. Please ensure your password meets these requirements and try again."
        exit 1
      fi
      pguser=$(get_ini_value "pguser")
    else
      echo "The supplied argument is not a .ini file. Exiting XFabric Agent Installer."
      exit 1
    fi
    if [ -z $pguser ]; then
      pguser="cyberpgsql"
    fi

    local composefile="docker-compose-@agentName@.yaml"
    create_docker_compose_file
    populate_extra_hosts

    #TODO: Uncomment below two line and rm line after the docker  compose up line. This is for
    #development environments to use the jfrog image
    curl -O https://repo.fortisoar.fortinet.com/xf/@agentVersion@/agent/docker/xf-agent-integration-@agentVersion@-@agentBuildNumber@.tgz
    docker load --input ./xf-agent-integration-@agentVersion@-@agentBuildNumber@.tgz
    dbpass=$pgpw dbuser=$pguser docker compose --project-name agent-@agentName@ -f $composefile up --detach
    rm -f ./xf-agent-integration-@agentVersion@-@agentBuildNumber@.tgz
}

main $@